/******************************************************************************
 * $Id: mappdf.c 7585 2008-05-13 05:19:24Z tbonfort $
 *
 * Project:  MapServer
 * Purpose:  An api for PDF format output using the pdflib or libharu library
 *           http://www.pdflib.com
 *           http://libharu.org
 * Author:   <kyngchaos@kyngchaos.com>
 *
 ******************************************************************************
 * Copyright (c) 1996-2005 Regents of the University of Minnesota.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies of this Software or works derived from this Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 ****************************************************************************/

#ifdef USE_PDF

/* Define common functions and constants according to PDF library used   */
/* to reduce conditionalizing in mappdf.c.  Most of these have the exact */
/* same parameters and order thereof, except the DRAWOBJ.  More could be */
/* done with macro function mangling like MAPPDF_FN_TEXT.                */

/*** libHaru ***/
#if defined USE_HARUPDF

/* PDF object functions */
#define MAPPDF_DOC HPDF_Doc
#define MAPPDF_DOC_TYPE HPDF_Doc
#define MAPPDF_FREE HPDF_Free
#define MAPPDF_FN_SETINFO HPDF_SetInfoAttr
#define MAPPDF_INFO_CREATOR HPDF_INFO_CREATOR
#define MAPPDF_INFO_AUTHOR HPDF_INFO_AUTHOR
#define MAPPDF_INFO_TITLE HPDF_INFO_TITLE

/* all draw operations on page object */
#define MAPPDF_DRAWOBJ page
#define MAPPDF_DRAWOBJ_TYPE HPDF_Page
#define MAPPDF_FN_GSAVE HPDF_Page_GSave
#define MAPPDF_FN_GRESTORE HPDF_Page_GRestore

/* path attrib functions */
#define MAPPDF_FN_SETLINEJOIN HPDF_Page_SetLineJoin
#define MAPPDF_FN_SETLINEWIDTH HPDF_Page_SetLineWidth
#define MAPPDF_FN_SETRGBSTROKECOLOR HPDF_Page_SetRGBStroke
#define MAPPDF_FN_SETRGBFILLCOLOR HPDF_Page_SetRGBFill
#define MAPPDF_LINEJOIN_MITER HPDF_MITER_JOIN
#define MAPPDF_LINEJOIN_ROUND HPDF_ROUND_JOIN

/* path draw functions */
/* #define MAPPDF_PDFY(y) HPDF_Page_GetHeight(page) - (y) */
#define MAPPDF_FN_MOVETO HPDF_Page_MoveTo
#define MAPPDF_FN_LINETO HPDF_Page_LineTo
#define MAPPDF_FN_CIRCLE HPDF_Page_Circle
#define MAPPDF_FN_STROKE HPDF_Page_Stroke
#define MAPPDF_FN_FILL HPDF_Page_Fill
#define MAPPDF_FN_FILLSTROKE HPDF_Page_FillStroke
#define MAPPDF_FN_CLOSEFILLSTROKE HPDF_Page_ClosePathFillStroke

/* text functions */
#define MAPPDF_FN_SETFONT HPDF_Page_SetFontAndSize
#define MAPPDF_FN_TEXTXY(p, s, x, y) HPDF_Page_TextOut((p), (x), (y), (s))
#define MAPPDF_FN_TEXTNEXTLINE HPDF_Page_ShowTextNextLine

/*** PDFlib ***/
#elif defined USE_PDFLIB

/* PDF object functions */
#define MAPPDF_DOC PDF
#define MAPPDF_DOC_TYPE PDF *
#define MAPPDF_FREE PDF_delete
#define MAPPDF_FN_SETINFO PDF_set_info
#define MAPPDF_INFO_CREATOR "Creator"
#define MAPPDF_INFO_AUTHOR "Author"
#define MAPPDF_INFO_TITLE "Title"

/* all draw operations on PDF object */
#define MAPPDF_DRAWOBJ pdf
#define MAPPDF_DRAWOBJ_TYPE PDF *
#define MAPPDF_FN_GSAVE PDF_save
#define MAPPDF_FN_GRESTORE PDF_restore

/* path attrib functions */
#define MAPPDF_FN_SETLINEJOIN PDF_setlinejoin
#define MAPPDF_FN_SETLINEWIDTH PDF_setlinewidth
#define MAPPDF_FN_SETRGBSTROKECOLOR PDF_setrgbcolor_stroke
#define MAPPDF_FN_SETRGBFILLCOLOR PDF_setrgbcolor_fill
#define MAPPDF_LINEJOIN_MITER 0
#define MAPPDF_LINEJOIN_ROUND 1

/* path draw functions */
/* #define MAPPDF_PDFY(y) (y) */
#define MAPPDF_FN_MOVETO PDF_moveto
#define MAPPDF_FN_LINETO PDF_lineto
#define MAPPDF_FN_CIRCLE PDF_circle
#define MAPPDF_FN_STROKE PDF_stroke
#define MAPPDF_FN_FILL PDF_fill
#define MAPPDF_FN_FILLSTROKE PDF_fill_stroke
#define MAPPDF_FN_CLOSEFILLSTROKE PDF_closepath_fill_stroke

/* text functions */
#define MAPPDF_FN_SETFONT PDF_setfont
#define MAPPDF_FN_TEXTXY(p, s, x, y) PDF_show_xy((p), (s), (x), (y))
#define MAPPDF_FN_TEXTNEXTLINE PDF_continue_text

#endif

#endif
